@@ -50,6 +50,7 @@ from mech_client.interact import (
     get_event_signatures,
     get_mech_config,
 )
+from mech_client.mech_marketplace_tool_management import get_mech_tools
 from mech_client.prompt_to_ipfs import push_metadata_to_ipfs
 from mech_client.wss import (
     register_event_handlers,
@@ -74,6 +75,8 @@ class PaymentType(Enum):
 
 
 IPFS_URL_TEMPLATE = "https://gateway.autonolas.tech/ipfs/f01701220{}"
+MECH_OFFCHAIN_REQUEST_ENDPOINT = "send_signed_requests"
+MECH_OFFCHAIN_DELIVER_ENDPOINT = "fetch_offchain_info"
 ABI_DIR_PATH = Path(__file__).parent / "abis"
 IMECH_ABI_PATH = ABI_DIR_PATH / "IMech.json"
 ITOKEN_ABI_PATH = ABI_DIR_PATH / "IToken.json"
@@ -252,6 +255,9 @@ def fetch_requester_nvm_subscription_balance(
     nvm_balance_tracker_contract = get_contract(
         contract_address=mech_payment_balance_tracker, abi=abi, ledger_api=ledger_api
     )
+    requester_balance_tracker_balance = (
+        nvm_balance_tracker_contract.functions.mapRequesterBalances(requester).call()
+    )
     subscription_nft_address = (
         nvm_balance_tracker_contract.functions.subscriptionNFT().call()
     )
@@ -269,7 +275,7 @@ def fetch_requester_nvm_subscription_balance(
         requester, subscription_id
     ).call()
 
-    return requester_balance
+    return requester_balance_tracker_balance + requester_balance
 
 
 def send_marketplace_request(  # pylint: disable=too-many-arguments,too-many-locals
@@ -389,6 +395,7 @@ def send_marketplace_request(  # pylint: disable=too-many-arguments,too-many-loc
 def send_offchain_marketplace_request(  # pylint: disable=too-many-arguments,too-many-locals
     crypto: EthereumCrypto,
     marketplace_contract: Web3Contract,
+    mech_offchain_url: str,
     prompt: str,
     tool: str,
     method_args_data: MechMarketplaceRequestConfig,
@@ -405,6 +412,8 @@ def send_offchain_marketplace_request(  # pylint: disable=too-many-arguments,too
     :type crypto: EthereumCrypto
     :param marketplace_contract: The mech marketplace contract instance.
     :type marketplace_contract: Web3Contract
+    :param mech_offchain_url: mech url to connect to.
+    :type mech_offchain_url: str
     :param prompt: The request prompt.
     :type prompt: str
     :param tool: The requested tool.
@@ -469,9 +478,9 @@ def send_offchain_marketplace_request(  # pylint: disable=too-many-arguments,too
                 "nonce": nonce,
                 "ipfs_data": ipfs_data,
             }
-            # @todo changed hardcoded url
+            url = mech_offchain_url + MECH_OFFCHAIN_REQUEST_ENDPOINT
             response = requests.post(
-                "http://localhost:8000/send_signed_requests",
+                url=url,
                 data=payload,
                 headers={"Content-Type": "application/json"},
             ).json()
@@ -562,10 +571,12 @@ def wait_for_marketplace_data_url(  # pylint: disable=too-many-arguments, unused
     return result
 
 
-def wait_for_offchain_marketplace_data(request_id: str) -> Any:
+def wait_for_offchain_marketplace_data(mech_offchain_url: str, request_id: str) -> Any:
     """
     Watches for data off-chain on mech.
 
+    :param mech_offchain_url: mech url to connect to.
+    :type mech_offchain_url: str
     :param request_id: The ID of the request.
     :type request_id: str
     :return: The data returned by the mech.
@@ -573,15 +584,17 @@ def wait_for_offchain_marketplace_data(request_id: str) -> Any:
     """
     while True:
         try:
-            # @todo change hardcoded url
+            url = mech_offchain_url + MECH_OFFCHAIN_DELIVER_ENDPOINT
             response = requests.get(
-                "http://localhost:8000/fetch_offchain_info",
+                url=url,
                 data={"request_id": request_id},
             ).json()
             if response:
                 return response
+
+            time.sleep(WAIT_SLEEP)
         except Exception:  # pylint: disable=broad-except
-            time.sleep(1)
+            time.sleep(WAIT_SLEEP)
 
 
 def check_prepaid_balances(
@@ -633,11 +646,36 @@ def check_prepaid_balances(
             sys.exit(1)
 
 
+def verify_tools(tools: tuple, service_id: int, chain_config: Optional[str]) -> None:
+    """
+    Verifies user supplied tool(s) with the mech's metadata
+
+    :param tools: The user supplied tools.
+    :type tools: tuple
+    :param service_id: Service id of the mech.
+    :type service_id: int
+    :param chain_config: Id of the mech's chain configuration (stored configs/mechs.json)
+    :type chain_config: str
+    :rtype: None
+    """
+    mech_tools_data = get_mech_tools(service_id=service_id, chain_config=chain_config)
+    if not mech_tools_data:
+        raise ValueError("Error while fetching mech tools data")
+
+    mech_tools = mech_tools_data.get("tools", [])
+    invalid_tools = [tool for tool in tools if tool not in mech_tools]
+    if invalid_tools:
+        raise ValueError(
+            f"Tool(s) {invalid_tools} not found in mech tools: {mech_tools}"
+        )
+
+
 def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals, too-many-statements, too-many-return-statements
     prompts: tuple,
     priority_mech: str,
     use_prepaid: bool = False,
     use_offchain: bool = False,
+    mech_offchain_url: str = "",
     tools: tuple = (),
     extra_attributes: Optional[Dict[str, Any]] = None,
     private_key_path: Optional[str] = None,
@@ -658,6 +696,8 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
     :type use_prepaid: bool
     :param use_offchain: Whether to use offchain model or not.
     :type use_offchain: bool
+    :param mech_offchain_url: mech url to connect to.
+    :type mech_offchain_url: str
     :param tools: The tools to interact with (optional).
     :type tools: tuple
     :param extra_attributes: Extra attributes to be included in the request metadata (optional).
@@ -732,7 +772,7 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
     )
     (
         payment_type,
-        _,
+        service_id,
         max_delivery_rate,
         mech_payment_balance_tracker,
         mech_contract,
@@ -744,6 +784,8 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
     mech_marketplace_request_config.delivery_rate = max_delivery_rate
     mech_marketplace_request_config.payment_type = payment_type
 
+    verify_tools(tools, service_id, chain_config)
+
     with open(IMECH_ABI_PATH, encoding="utf-8") as f:
         abi = json.load(f)
 
@@ -792,22 +834,29 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
             max_delivery_rate,
         )
 
-    if payment_type in [PaymentType.NATIVE_NVM.value, PaymentType.TOKEN_NVM.value]:
+    is_nvm_mech = payment_type in [
+        PaymentType.NATIVE_NVM.value,
+        PaymentType.TOKEN_NVM.value,
+    ]
+    if is_nvm_mech:
         nvm_mech_type = PaymentType(payment_type).name.lower()
         print(
             f"{nvm_mech_type} Nevermined Mech detected, subscription credits to be used"
         )
         requester = crypto.address
-        requester_balance = fetch_requester_nvm_subscription_balance(
+        requester_total_balance_before = fetch_requester_nvm_subscription_balance(
             requester, ledger_api, mech_payment_balance_tracker, payment_type
         )
-        if requester_balance < price:
+        if requester_total_balance_before < price:
             print(
-                f"  - Sender Subscription balance low. Needed: {price}, Actual: {requester_balance}"
+                f"  - Sender Subscription balance low. Needed: {price}, Actual: {requester_total_balance_before}"
             )
             print(f"  - Sender Address: {requester}")
             sys.exit(1)
 
+        print(
+            f"  - Sender Subscription balance before request: {requester_total_balance_before}"
+        )
         # set price 0 to not send any msg.value in request transaction for nvm type mech
         price = 0
 
@@ -868,6 +917,19 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
             )
 
             if data_url:
+                if is_nvm_mech:
+                    requester_total_balance_after = (
+                        fetch_requester_nvm_subscription_balance(
+                            requester,
+                            ledger_api,
+                            mech_payment_balance_tracker,
+                            payment_type,
+                        )
+                    )
+                    print(
+                        f"  - Sender Subscription balance after delivery: {requester_total_balance_after}"
+                    )
+
                 print(f"  - Data arrived: {data_url}")
                 data = requests.get(f"{data_url}/{request_id_int}", timeout=30).json()
                 print("  - Data from agent:")
@@ -882,6 +944,7 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
         response = send_offchain_marketplace_request(
             crypto=crypto,
             marketplace_contract=mech_marketplace_contract,
+            mech_offchain_url=mech_offchain_url,
             prompt=prompts[0],
             tool=tools[0],
             method_args_data=mech_marketplace_request_config,
@@ -910,6 +973,7 @@ def marketplace_interact(  # pylint: disable=too-many-arguments, too-many-locals
 
     for request_id in request_ids:
         data = wait_for_offchain_marketplace_data(
+            mech_offchain_url=mech_offchain_url,
             request_id=request_id,
         )
 
